@@ -112,7 +112,8 @@ static const char *four_hundred[] = {
   "Unknown",
   "Unknown",
   "Retry With",
-  "Blocked by Windows Parental Controls" /* 450 */
+  "Blocked by Windows Parental Controls", /* 450 */
+  "Unavailable For Legal Reasons"
 };
 
 static const char *five_hundred[] = {
@@ -150,7 +151,9 @@ static const struct MHD_Reason_Block reasons[] = {
 const char *
 MHD_get_reason_phrase_for (unsigned int code)
 {
-  if ((code >= 100 && code < 600) && (reasons[code / 100].max > code % 100))
+  if ( (code >= 100) && 
+       (code < 600) && 
+       (reasons[code / 100].max > (code % 100)) )
     return reasons[code / 100].data[code % 100];
   return "Unknown";
 }
